package org.eclipse.jdt.internal.compiler.batch;

import java.util.Iterator;
import org.eclipse.jdt.core.compiler.CharOperation;
import java.util.ArrayList;
import java.io.InputStream;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFormatException;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFileReader;
import java.io.IOException;
import java.io.File;
import org.eclipse.jdt.internal.compiler.env.NameEnvironmentAnswer;
import java.util.List;
import java.util.HashSet;
import javax.tools.JavaFileManager;
import javax.tools.JavaFileObject;
import java.util.Set;

public class ClasspathJsr199 extends ClasspathLocation
{
    private static final Set<JavaFileObject.Kind> fileTypes;
    private JavaFileManager fileManager;
    private JavaFileManager.Location location;
    
    static {
        (fileTypes = new HashSet()).add(JavaFileObject.Kind.CLASS);
    }
    
    public ClasspathJsr199(JavaFileManager file, JavaFileManager.Location location) {
        super(null, null);
        this.fileManager = file;
        this.location = location;
    }
    
    @Override
    public List fetchLinkedJars(FileSystem.ClasspathSectionProblemReporter problemReporter) {
        return null;
    }
    
    @Override
    public NameEnvironmentAnswer findClass(char[] typeName, String qualifiedPackageName, String qualifiedBinaryFileName) {
        return this.findClass(typeName, qualifiedPackageName, qualifiedBinaryFileName, false);
    }
    
    @Override
    public NameEnvironmentAnswer findClass(char[] typeName, String qualifiedPackageName, String aQualifiedBinaryFileName, boolean asBinaryOnly) {
        String qualifiedBinaryFileName = (File.separatorChar == '/') ? aQualifiedBinaryFileName : aQualifiedBinaryFileName.replace(File.separatorChar, '/');
        try {
            int lastDot = qualifiedBinaryFileName.lastIndexOf(46);
            String className = (lastDot < 0) ? qualifiedBinaryFileName : qualifiedBinaryFileName.substring(0, lastDot);
            JavaFileObject jfo = null;
            try {
                jfo = this.fileManager.getJavaFileForInput(this.location, className, JavaFileObject.Kind.CLASS);
            }
            catch (IOException ex) {}
            if (jfo == null) {
                return null;
            }
            Throwable t = null;
            try {
                InputStream inputStream = jfo.openInputStream();
                try {
                    ClassFileReader reader = ClassFileReader.read(inputStream, qualifiedBinaryFileName);
                    if (reader != null) {
                        NameEnvironmentAnswer nameEnvironmentAnswer = new NameEnvironmentAnswer(reader, this.fetchAccessRestriction(qualifiedBinaryFileName));
                    }
                }
                finally {
                    if (inputStream != null) {
                        inputStream.close();
                    }
                }
            }
            finally {
                if (t == null) {
                    Throwable exception;
                    t = exception;
                }
                else {
                    Throwable exception;
                    if (t != exception) {
                        t.addSuppressed(exception);
                    }
                }
            }
        }
        catch (ClassFormatException ex2) {}
        catch (IOException ex3) {}
        return null;
    }
    
    @Override
    public char[][][] findTypeNames(String aQualifiedPackageName) {
        String qualifiedPackageName = (File.separatorChar == '/') ? aQualifiedPackageName : aQualifiedPackageName.replace(File.separatorChar, '/');
        Iterable<JavaFileObject> files = null;
        try {
            files = this.fileManager.list(this.location, qualifiedPackageName, ClasspathJsr199.fileTypes, false);
        }
        catch (IOException ex) {}
        if (files == null) {
            return null;
        }
        ArrayList<char[][]> answers = new ArrayList();
        char[][] packageName = CharOperation.splitOn(File.separatorChar, qualifiedPackageName.toCharArray());
        for (JavaFileObject file : files) {
            String fileName = file.toUri().getPath();
            int last = fileName.lastIndexOf(47);
            if (last > 0) {
                int indexOfDot = fileName.lastIndexOf(46);
                if (indexOfDot != -1) {
                    String typeName = fileName.substring(last + 1, indexOfDot);
                    answers.add(CharOperation.arrayConcat(packageName, typeName.toCharArray()));
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        int size = answers.size();
        if (size != 0) {
            char[][][] result = new char[size][][];
            answers.toArray(result);
            return result;
        }
        return null;
    }
    
    @Override
    public void initialize() throws IOException {
    }
    
    @Override
    public boolean isPackage(String aQualifiedPackageName) {
        String qualifiedPackageName = (File.separatorChar == '/') ? aQualifiedPackageName : aQualifiedPackageName.replace(File.separatorChar, '/');
        boolean result = false;
        try {
            Iterable<JavaFileObject> files = this.fileManager.list(this.location, qualifiedPackageName, ClasspathJsr199.fileTypes, false);
            Iterator<JavaFileObject> f = files.iterator();
            if (f.hasNext()) {
                result = true;
            }
            else {
                files = this.fileManager.list(this.location, qualifiedPackageName, ClasspathJsr199.fileTypes, true);
                f = files.iterator();
                if (f.hasNext()) {
                    result = true;
                }
            }
        }
        catch (IOException ex) {}
        return result;
    }
    
    @Override
    public void reset() {
        try {
            this.fileManager.flush();
        }
        catch (IOException ex) {}
    }
    
    @Override
    public String toString() {
        return "Classpath for Jsr 199 JavaFileManager: " + this.location;
    }
    
    @Override
    public char[] normalizedPath() {
        if (this.normalizedPath == null) {
            this.normalizedPath = this.path.toCharArray();
        }
        return this.normalizedPath;
    }
    
    @Override
    public String getPath() {
        if (this.path == null) {
            this.path = this.location.getName();
        }
        return this.path;
    }
    
    @Override
    public int getMode() {
        return 2;
    }
    
    @Override
    public boolean hasAnnotationFileFor(String qualifiedTypeName) {
        return false;
    }
}
